function [ S2]  =  NextState_Tip( S1, a1, t1, deltaT1, JJ )
%Finds the next state given the current state and actions
%   S1: Initial state (length 7) at time t
%   act: action vector
%       act(1): saving rate
%       act(2): GHG reduction rate
%   S2: Next states
%       S2(1:7): Next state

global gama sai1 sai2 sai3 deltak theta1 theta2
global Sig Eland b11 b12 b21 b22 b23 b32 b33 pai
global etha1 etha3 etha4 deltarf Fex L A

S2 = zeros(7, 1);

%Industrial Emission
IE = Sig(t1) * (1 - a1) * S1(7);

%Total Emission
E = IE + Eland(t1);

%CO2 doubling coefficient
etha2 = deltarf / deltaT1;

%Lower Ocean Concentration
S2(6) = b23 / 100 * S1(5) + b33 / 100 * S1(6);

%Upper Ocean Concentration
S2(5) = b12 / 100 * S1(4) + b22 / 100 * S1(5)+ b32 / 100 * S1(6);

%Atmospheric Concentration
S2(4) = 10 * E + b11 / 100 * S1(4) + b21 / 100 * S1(5);

%Lower Ocean Temperature
S2(3) = S1(3) + etha4 * (S1(2) - S1(3));

%Radiative Forcings
F = deltarf * ((log(S2(4)) - log(596.4)) / log(2)) + Fex(t1 + 1);

%Atmospheric Temperature
S2(2) = S1(2) + etha1 * (F - etha2 * S1(2) - etha3 * (S1(2) - S1(3)));

Damage = (1 - JJ) / (1 + sai1 * S1(2) + sai2 * S1(2) ^ sai3);
Abate = pai(t1) ^ (1 - theta2) * theta1(t1) * a1 ^ theta2;

%Economic output net of damage and abatement costs
Q = (Damage - Abate) * S1(7);

%Capital at time t1+1
S2(1) = 10 * 0.22 * Q + (1 - deltak) ^ 10 * S1(1);

%Gross output at time t1+1
S2(7) = A(t1+1) * (S2(1) ^ gama) * L(t1 + 1)^(1 - gama);

end

