clc
clear;
Initialset;

global  k0 Tat0 Tlo0 Mat0 Mlo0 Mup0 a0 dt_l dt_u dt_m dt_sig
global  S L A T gama lambda nu x deltaT_pdf action J1

%   Number of runs
    mmax = 1000;
   
    n10 = 1;
    n60 = 1;
    S10 = zeros(mmax * 8, T);
    S60 = zeros(mmax * 8, T);
    m10 = zeros (mmax, 1);
    m60 = zeros (mmax, 1);
    
%	Uncertainty in Climate parameters    
    % Climate sensitivity
    dt_l = 0;
    dt_u = 20;    
    dt_m = 1.1;
    dt_sig = 0.55;
    
    deltaT0 = 3;
    deltaT_path = ones(mmax, T);   
    deltaT_pdf = zeros(100, T);
    deltaT_cdf = zeros(100, T);
    
    % Damage function
    x = zeros(mmax, T);
    pt = zeros(mmax, T);
    J0 = 0;
    J1 = 0.025 * 10;
    nu = 0.006 * 10;  
   
%   State space:
    S = zeros(7, T);
    %S(1, t): Capital "K" at time t
    %S(2, t): Atmospheric Temperature "Tat" at time t
    %S(3, t): Lower ocean Temperature "Tlo" at time t
    %S(4, t): Atmospheric Carbon Concentration "Mat" at time t
    %S(5, t): Upper Ocean Carbon Concentration "Mup" at time t
    %S(6, t): Lower Ocean Carbon Concentration "Mlo" at time t
    %S(7, t): Gross Economic Output "Y" at time t
    
%   Optimal policy (GHG Reduction Rate %) "a" at state i at time t
    action = zeros(mmax, T); 

%	Approximate value "V-" of the post-decision stateat time t (Vbar)
    Vbar = zeros(1, T);

%	True value "V^" at time t (Vhat)
    Vhat = zeros(1, T + 1);

%	Error in estimating the Value Function    
    error = zeros(mmax, T);
    
%	Coefficient of aaproximate value function    
    coefm = ones(mmax , T);
    
%	Compounents of aaproximate value function
    phi = zeros(1, T);
    
%   Initial State
    y0 = A(1) * (k0 ^ gama) * L(1) ^ (1 - gama);
    S(:, 1) = [k0, Tat0, Tlo0, Mat0, Mlo0, Mup0, y0];

for m = 1:1:mmax    
        deltaT_path(m, 1) = lognorm_trunc(rand);
        for t = 1:1:T
            if (t == T)
                action(m, t) = 1;
                Vhat(t) = Utility_Tip(S(:, t), 1, t, x(m, t) * J1);
            else                
                Vm = 0;
                for i = 0:1:100
                    if (t == 1)
                        a = a0;
                    else
                        a = i/100;
                    end 
                    
                    %	Value of the Post-Decision state                
                    if (x(m, t) == 0)
                        pt1 = 1 - exp(-nu * max(0, (S(2, t) - 1)));
                        J = J0;
                    else
                        pt1 = 1;
                        J = J1;
                    end
                    
                    adj = floor((t - sum(x(m,:)))/10) * 0.2;
                    deltaT1 = deltaT0 - adj;
                    Sa1 = NextState_Tip(S(:, t), a, t, deltaT1, J);                      
                    U1 = (1 - pt1) * Utility_Tip(Sa1, 0, t + 1, J0) + pt1 * Utility_Tip(Sa1, 0, t + 1, J1);
                    
                    if (t < T-1)
                        
                        adj20 = floor((t + 1 - sum(x(m,:)))/10) * 0.2;
                        deltaT20 = deltaT0 - adj20;
                        Sa20 = NextState_Tip(Sa1, 0, t + 1, deltaT20, J0);
                        
                        adj21 = floor((t + 1 - (sum(x(m,:)) + 1))/10) * 0.2;
                        deltaT21 = deltaT0 - adj21;                                               
                        Sa21 = NextState_Tip(Sa1, 0, t + 1, deltaT21, J1);
                        
                        pt2 = 1 - exp(-nu * max(0, (Sa1(2) - 1)));
                        U2 = (1 - pt1) * ((1 - pt2) * Utility_Tip(Sa20, 0, t + 2, J0) + pt2 * Utility_Tip(Sa20, 0, t + 2, J1)) + pt1 * Utility_Tip(Sa21, 0, t + 2, J1);
                    else
                        U2 = U1;
                    end

                    Vb = coefm(m, t) * U2;

                    %	utility of the current state
                    W = Utility_Tip(S(:, t), a, t, J);

                    %   Approximate value of the current state
                    V = W + lambda * Vb;

                    if (Vm < V )
                        Vm = V;
                        Vhat(t) = V;
                        phi(t) = U2;
                        Vbar(t) = coefm(m, t) * phi(t);
                        action(m, t) = a;
                        pt(m,t) = pt1;
                    end
                    if (t == 1)
                        break
                    end
                end
                
                %	Finding the next coefficient of the Damage function
                if (x(m, t) ~= 0)
                    x(m, t + 1) = x(m, t);
                elseif (pt(m,t) > rand)
                    x(m, t + 1) = 1;
                else
                    x(m, t + 1) = 0;
                end
                %   Bayesian update for deltaT
                if (t == 1)
                    for i = 1:1:100
                        yi = dt_l + (dt_u - dt_l) / 99 * (i - 1);
                        deltaT_cdf(i, t) = lognorm_trunc_inv(yi);
                        if (i > 1)
                            deltaT_pdf(i, t) = deltaT_cdf(i, t) - deltaT_cdf(i - 1, t);
                        else
                            deltaT_pdf(i, t) = deltaT_cdf(i, t);
                        end     
                    end
                    pri_pdf = deltaT_pdf(:, t);
                else               
                    pri_pdf = deltaT_pdf(:, t - 1);
                end
                [post_pdf post_cdf deltaT_rand] = deltaT_update(pri_pdf, m, t, rand);
                deltaT_pdf(:, t) = post_pdf;
                deltaT_cdf(:, t) = post_cdf;
                deltaT_path(m, t + 1) = deltaT_rand;
                
                %	Finding the next pre-decision state
                S(:, t + 1) = NextState_Tip(S(:, t), action(m, t), t, deltaT_path(m, t + 1), J); 
            end
            %	Updating coefficient of approximate value function
            if (t > 1)    
                %	Value approximation Error
                error(m, t - 1) = Vbar(t - 1) - Vhat(t);

                %   Bellman Update
                lm = 1/1 * phi(t - 1)^-2;
                coefm(m + 1, t - 1) = coefm(m, t - 1) - lm * error(m, t - 1) * phi(t - 1);
            end
        end
        if (x(m, 10) == 1 && x(m,9) == 0)            
            S10((n10 - 1) * 8 + 1:(n10 - 1) * 8 + 7, :) = S;
            m10(n10) = m;
            n10 = n10 + 1;
        elseif x(m, 60) == 0
            deltaT_pdf_NT = deltaT_pdf(:, :);
            S60((n60 - 1) * 8 + 1:(n60 - 1) * 8 + 7, :) = S;
            m60(n60) = m;
            n60 = n60 + 1;
        end       
end
