clc
clear;
Initialset;

global  k0 Tat0 Tlo0 Mat0 Mlo0 Mup0 a0 dt_l dt_u dt_m dt_sig
global  S L A T gama lambda

%   Number of runs
    mmax = 1000;
   
%	Uncertainty in Climate Sensitivity 
    dt_l = 0.1;
    dt_u = 20;
    dt_m = 1.1;
    dt_sig = 0.55;
    deltaT = 3;
    
    deltaT_path = ones(mmax, T); 
    
%   State space:
    S = zeros(7, T);
    %S(1, t): Capital "K" at time t
    %S(2, t): Atmospheric Temperature "Tat" at time t
    %S(3, t): Lower ocean Temperature "Tlo" at time t
    %S(4, t): Atmospheric Carbon Concentration "Mat" at time t
    %S(5, t): Upper Ocean Carbon Concentration "Mup" at time t
    %S(6, t): Lower Ocean Carbon Concentration "Mlo" at time t
    %S(7, t): Gross Economic Output "Y" at time t
    
%   Optimal policy (GHG Reduction Rate %) "a" at state i at time t
    action = zeros(mmax, T); 

%	Approximate value "V-" of the post-decision stateat time t (Vbar)
    Vbar = zeros(1, T);

%	True value "V^" at time t (Vhat)
    Vhat = zeros(1, T + 1);

%	Error in estimating the Value Function    
    error = zeros(mmax, T);
    
%	Coefficient of aaproximate value function    
    coefm = zeros(mmax + 1, T);
    
%	Compounents of aaproximate value function
    phi = zeros(1, T); 
    
%   Initial State
    y0 = A(1) * (k0 ^ gama) * L(1) ^ (1 - gama);
    S(:, 1) = [k0, Tat0, Tlo0, Mat0, Mlo0, Mup0, y0];
    
    
for m = 1:1:mmax    
        %	Generating a sample path based on truncated-lognormal distribution
        urand = rand(1, T);
        deltaT_path(m, 1:T) = lognorm_trunc( urand );
        
        for t = 1:1:T
            if (t == T)
                action(m, t) = 1;
                Vhat(t) = Utility(S(1:7, t), 1, t);
            else    
                Vm = 0;
                for i = 0:1:100
                    if (t == 1)
                        a = a0;
                    else
                        a = i/100;
                    end
                    
                    %	Value of the Post-Decision state
                    Sa1 = NextState(S(1:7, t), a, t, deltaT);
                    U1 = Utility(Sa1, 0, t + 1);
                    if (t < T-1)
                        Sa2 = NextState(Sa1, 0, t + 1, deltaT);
                        U2 = Utility(Sa2, 0, t + 2);
                    else
                        U2 = U1;
                    end

                    Vb = coefm(m, t) * U2;

                    %	utility of the current state
                    W = Utility(S(1:7, t), a, t);

                    %   Approximate value of the current state
                    V = W + lambda * Vb;

                    if (Vm < V )
                        Vm = V;
                        Vhat(t) = V;
                        phi(t) = U2;
                        Vbar(t) = coefm(m, t) * phi(t);
                        action(m, t) = a;
                    end
                    if (t == 1)
                        break
                    end
                end

                %	Finding the next pre-decision state
                S(:, t + 1) = NextState(S(:, t), action(m, t), t, deltaT_path(m, t + 1)); 
            end
            
            %	Updating coefficient of approximate value function
            if (t > 1)    
                %	Value approximation Error
                error(m, t - 1) = Vbar(t - 1) - Vhat(t);

                %   Bellman Update
                lm = 1/1 * phi(t - 1)^-2;
                coefm(m + 1, t - 1) = coefm(m, t - 1) - lm * error(m, t - 1) * phi(t - 1);
            end
        end
end
