%%%%% %%%%% %%%%% %%%%% %%%%% %%%%% %%%%% %%%%% %%%%% %%%%% %%%%% %%%%%
%%%%% This code calculates the optimal control rate for DICE 2007 %%%%%
%%%%% Climate sensitivity: fixed                                  %%%%%
%%%%% Tipping point event: Uncertain                              %%%%%
%%%%% %%%%% %%%%% %%%%% %%%%% %%%%% %%%%% %%%%% %%%%% %%%%% %%%%% %%%%%

clc
clear;

%   Initialization
Initialset;

global  k0 Tat0 Tlo0 Mat0 Mlo0 Mup0 a0
global  S L A T gama lambda

%   Number of runs
    mmax = 1000;
   
%	Uncertainty in Climate parameters    
    % Climate sensitivity is fxed
    deltaT = 3;  
    
    % Damage function
    x = zeros(mmax, T);
    pt = zeros(mmax, T);
    J0 = 0;
    J1 = 0.025 * 10;
    nu = 0.006 * 10;  
   
%   State space:
    S = zeros(7, T);
    %S(1, t): Capital "K" at time t
    %S(2, t): Atmospheric Temperature "Tat" at time t
    %S(3, t): Lower ocean Temperature "Tlo" at time t
    %S(4, t): Atmospheric Carbon Concentration "Mat" at time t
    %S(5, t): Upper Ocean Carbon Concentration "Mup" at time t
    %S(6, t): Lower Ocean Carbon Concentration "Mlo" at time t
    %S(7, t): Gross Economic Output "Y" at time t
    
%   Optimal policy (Control Rate %) in state i at time t
    action = zeros(mmax, T); 

%	Approximate value "V-" of the post-decision stateat time t
    Vbar = zeros(1, T);

%	True value "V^" at time t
    Vhat = zeros(1, T + 1);

%	Error in Value Function Estimation 
    error = zeros(mmax, T);
    
%	Coefficient of aaproximate value function    
    coefm = ones(mmax , T);
    
%	Compounents of aaproximate value function
    phi = zeros(1, T);
    
%   Initial State
    y0 = A(1) * (k0 ^ gama) * L(1) ^ (1 - gama);
    S(:, 1) = [k0, Tat0, Tlo0, Mat0, Mlo0, Mup0, y0];    

for m = 1:1:mmax
    for t = 1:1:T
        %   Boundary conditions in the terminal state 
        if (t == T)
            action(m, t) = 1;
            Vhat(t) = Utility_Tip(S(:, t), 1, t, x(m, t) * J1);
        else
            Vm = 0;
            for i = 0:1:100
                if (t == 1)
                    a = a0;
                else
                    a = i/100;
                end
                
                %	Value of the Post-Decision state
                if (x(m, t) == 0)
                    %	Probability of tipping point event 
                    pt1 = 1 - exp(-nu * max(0, (S(2, t) - 1)));
                    J = J0;
                else
                    pt1 = 1;
                    J = J1;
                end
                %   one-step-ahead post-decision state
                Sa1 = NextState_Tip(S(:, t), a, t, deltaT, J);                      
                U1 = (1 - pt1) * Utility_Tip(Sa1, 0, t + 1, J0) + pt1 * Utility_Tip(Sa1, 0, t + 1, J1);                      
                if (t < T-1)
                    %   two-step-ahead post-decision states
                    Sa20 = NextState_Tip(Sa1, 0, t + 1, deltaT, J0);
                    Sa21 = NextState_Tip(Sa1, 0, t + 1, deltaT, J1);
                    pt2 = 1 - exp(-nu * max(0, (Sa1(2) - 1)));
                    U2 = (1 - pt1) * ((1 - pt2) * Utility_Tip(Sa20, 0, t + 2, J0) + pt2 * Utility_Tip(Sa20, 0, t + 2, J1)) + pt1 * Utility_Tip(Sa21, 0, t + 2, J1);                  
                else
                    U2 = U1;
                end
                
                Vb = coefm(m, t) * U2;

                %	utility of the current state
                W = Utility_Tip(S(:, t), a, t, J);

                %   Approximate value of the current state
                V = W + lambda * Vb;

                if (Vm < V )
                    Vm = V;
                    Vhat(t) = V;
                    phi(t) = U2;
                    Vbar(t) = coefm(m, t) * phi(t);
                    action(m, t) = a;
                    pt(m, t) = pt1;
                end
                if (t == 1)
                    break
                end
            end
                
            %	Tipping point status of the next state
            if (x(m, t) ~= 0)
                x(m, t + 1) = x(m, t);
            elseif (pt(m, t) > rand)
                x(m, t + 1) = 1;
            else
                x(m, t + 1) = 0;
            end
                    
            %	Finding the next pre-decision state
            S(:, t + 1) = NextState_Tip(S(:, t), action(m, t), t, deltaT, J); 
        end
        
        %	Updating coefficient of approximate value function
        if (t > 1)
            %	Value function approximation error
            error(m, t - 1) = Vbar(t - 1) - Vhat(t);

            %   Bellman Update
            lm = 1/1 * phi(t - 1)^-2;
            coefm(m + 1, t - 1) = coefm(m, t - 1) - lm * error(m, t - 1) * phi(t - 1);
        end
    end
    
    %   Storing the result for tipping point @ t=10
    if (x(m, 10)==1 && x(m,9)==0)
        S10 = S;
        m10 = m;
    %   Storing the result for no tipping point
    elseif x(m, 60)==0
        S60 = S;
        m60 = m;
    end
end
